@@ -131,7 +131,8 @@ summary.jm <- function (object, ...) {
                 families = families, respVars = respVars,
                 events = object$model_data$delta,
                 control = object$control, time = object$running_time,
-                call = object$call, recurrent = object$model_info$recurrent)
+                call = object$call, recurrent = object$model_info$recurrent,
+                any_terminal = length(object$model_data$which_term_h) > 0)
     tab_f <- function(name) {
         out <- data.frame(Mean = object$statistics$Mean[[name]],
                           StDev = object$statistics$SD[[name]],
@@ -162,7 +163,7 @@ summary.jm <- function (object, ...) {
             row.names(out[[nam_outcome]])[k + 1] <- "sigma"
         }
     }
-    if(out$recurrent) {
+    if(out$recurrent & out$any_terminal) {
       out$Survival <- do.call(rbind, list(tab_f("gammas"), tab_f("alphas"),
                                           tab_f("alphaF")))
     } else {
@@ -601,10 +602,12 @@ predict.jm <- function (object, newdata = NULL, newdata2 = NULL,
                         type = c("subject_specific", "mean_subject"),
                         level = 0.95, return_newdata = FALSE,
                         return_mcmc = FALSE, n_samples = 200L, n_mcmc = 55L,
+                        parallel = c("snow", "multicore"),
                         cores = NULL, seed = 123L, ...) {
     process <- match.arg(process)
     type_pred <- match.arg(type_pred)
     type <- match.arg(type)
+    parallel <- match.arg(parallel)
     id_var <- object$model_info$var_names$idVar
     time_var <- object$model_info$var_names$time_var
     Time_var <- object$model_info$var_names$Time_var
@@ -733,7 +736,7 @@ predict.jm <- function (object, newdata = NULL, newdata2 = NULL,
         cores <- if (n > 20) 4L else 1L
     }
     components_newdata <- get_components_newdata(object, newdata, n_samples,
-                                                 n_mcmc, cores, seed)
+                                                 n_mcmc, parallel, cores, seed)
     if (process == "longitudinal") {
         predict_Long(object, components_newdata, newdata, newdata2, times,
                      times_per_id, type, type_pred, level, return_newdata,
@@ -1049,15 +1052,18 @@ rc_setup <- function(rc_data, trm_data,
 
 
 predict.jmList <- function (object, weights, newdata = NULL, newdata2 = NULL,
-                        times = NULL, process = c("longitudinal", "event"),
+                        times = NULL, times_per_id = FALSE,
+                        process = c("longitudinal", "event"),
                         type_pred = c("response", "link"),
                         type = c("subject_specific", "mean_subject"),
                         level = 0.95, return_newdata = FALSE,
                         return_mcmc = FALSE, n_samples = 200L, n_mcmc = 55L,
-                        cores = max(parallel::detectCores() - 1, 1), ...) {
+                        parallel = c("snow", "multicore"),
+                        cores = parallelly::availableCores(omit = 1L), ...) {
     process <- match.arg(process)
     type_pred <- match.arg(type_pred)
     type <- match.arg(type)
+    parallel <- match.arg(parallel)
     obj <- object[[1L]]
     id_var <- obj$model_info$var_names$idVar
     time_var <- obj$model_info$var_names$time_var
@@ -1181,17 +1187,50 @@ predict.jmList <- function (object, weights, newdata = NULL, newdata2 = NULL,
                  "variable(s): ", paste(missing_vars, collapse = ", "), ".\n")
         }
     }
-    ncores <- min(cores, length(object))
-    cl <- parallel::makeCluster(ncores)
-    invisible(parallel::clusterEvalQ(cl, library("JMbayes2")))
-    preds <-
-        parallel::parLapply(cl, object, predict, newdata = newdata,
-                            newdata2 = newdata2, times = times,
-                            process = process, type_pred = type_pred,
-                            type = type, level = level, n_samples = n_samples,
-                            n_mcmc = n_mcmc, return_newdata = return_newdata,
-                            return_mcmc = TRUE)
-    parallel::stopCluster(cl)
+    cores <- min(cores, length(object))
+    if (cores > 1L) {
+        have_mc <- have_snow <- FALSE
+        if (parallel == "multicore") {
+            have_mc <- .Platform$OS.type != "windows"
+        } else if (parallel == "snow") {
+            have_snow <- TRUE
+        }
+        if (!have_mc && !have_snow) cores <- 1L
+        loadNamespace("parallel")
+    }
+    if (cores > 1L) {
+        if (have_mc) {
+            preds <-
+                parallel::mclapply(object, predict, newdata = newdata,
+                                   newdata2 = newdata2, times = times,
+                                   times_per_id = times_per_id,
+                                   process = process, type_pred = type_pred,
+                                   type = type, level = level, n_samples = n_samples,
+                                   n_mcmc = n_mcmc, return_newdata = return_newdata,
+                                   return_mcmc = TRUE, mc.cores = cores)
+        } else {
+            cl <- parallel::makePSOCKcluster(rep("localhost", cores))
+            invisible(parallel::clusterEvalQ(cl, library("JMbayes2")))
+            preds <-
+                parallel::parLapply(cl, object, predict, newdata = newdata,
+                                    newdata2 = newdata2, times = times,
+                                    times_per_id = times_per_id,
+                                    process = process, type_pred = type_pred,
+                                    type = type, level = level, n_samples = n_samples,
+                                    n_mcmc = n_mcmc, return_newdata = return_newdata,
+                                    return_mcmc = TRUE)
+            parallel::stopCluster(cl)
+        }
+    } else {
+        preds <-
+            lapply(object, predict, newdata = newdata,
+                   newdata2 = newdata2, times = times,
+                   times_per_id = times_per_id,
+                   process = process, type_pred = type_pred,
+                   type = type, level = level, n_samples = n_samples,
+                   n_mcmc = n_mcmc, return_newdata = return_newdata,
+                   return_mcmc = TRUE)
+    }
     extract_mcmc <- function (x) {
         if (is.data.frame(x)) attr(x, "mcmc") else x[["mcmc"]]
     }
