@@ -72,16 +72,29 @@ jm_fit <- function (model_data, model_info, initial_values, priors, control) {
     initial_values$alphas <- unlist(initial_values$alphas, use.names = FALSE)
     priors$mean_alphas <- unlist(priors$mean_alphas, use.names = FALSE)
     priors$Tau_alphas <- .bdiag(priors$Tau_alphas)
-    # random seed
-    if (!exists(".Random.seed", envir = .GlobalEnv))
-        runif(1)
-    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
-    on.exit(assign(".Random.seed", RNGstate, envir = .GlobalEnv))
+    priors$mean_alphaF <- unlist(priors$mean_alphaF, use.names = FALSE)
+    priors$Tau_alphaF <- .bdiag(priors$Tau_alphaF)
     n_chains <- control$n_chains
     tik <- proc.time()
     cores <- control$cores
+    parallel <- control$parallel
     chains <- seq_len(n_chains)
     cores <- min(cores, length(chains))
+    if (cores > 1L) {
+        have_mc <- have_snow <- FALSE
+        if (parallel == "multicore") {
+            have_mc <- .Platform$OS.type != "windows"
+        } else if (parallel == "snow") {
+            have_snow <- TRUE
+        }
+        if (!have_mc && !have_snow) cores <- 1L
+        loadNamespace("parallel")
+    }
+    # random seed
+    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
+        runif(1L)
+    RNGstate <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
+    on.exit(assign(".Random.seed", RNGstate, envir = .GlobalEnv))
     mcmc_parallel <- function (chain, model_data, model_info, initial_values,
                                priors, control) {
       not_D <- !names(initial_values) %in% c("D")
@@ -89,13 +102,23 @@ jm_fit <- function (model_data, model_info, initial_values, priors, control) {
       mcmc_cpp(model_data, model_info, initial_values, priors, control)
     }
     if (cores > 1L) {
-        cl <- parallel::makeCluster(cores)
-        parallel::clusterSetRNGStream(cl = cl, iseed = control$seed)
-        out <- parallel::parLapply(cl, chains, mcmc_parallel,
-                                   model_data = model_data, model_info = model_info,
-                                   initial_values = initial_values,
-                                   priors = priors, control = control)
-        parallel::stopCluster(cl)
+        if (have_mc) {
+            RNGkind("L'Ecuyer-CMRG")
+            set.seed(control$seed)
+            out <- parallel::mclapply(chains, mcmc_parallel,
+                                      model_data = model_data, model_info = model_info,
+                                      initial_values = initial_values,
+                                      priors = priors, control = control,
+                                      mc.cores = cores)
+        } else if (have_snow) {
+            cl <- parallel::makePSOCKcluster(rep("localhost", cores))
+            parallel::clusterSetRNGStream(cl = cl, iseed = control$seed)
+            out <- parallel::parLapply(cl, chains, mcmc_parallel,
+                                       model_data = model_data, model_info = model_info,
+                                       initial_values = initial_values,
+                                       priors = priors, control = control)
+            parallel::stopCluster(cl)
+        }
     } else {
         set.seed(control$seed)
         out <- lapply(chains, mcmc_parallel, model_data = model_data,
@@ -145,7 +168,12 @@ jm_fit <- function (model_data, model_info, initial_values, priors, control) {
             paste0("sigmas_",
                    seq_along(out[[i]][["mcmc"]][["sigmas"]][1, ]))
         colnames(out[[i]][["mcmc"]][["sigmaF"]]) <- "sigma_frailty"
-        colnames(out[[i]][["mcmc"]][["alphaF"]]) <- "frailty"
+        colnames(out[[i]][["mcmc"]][["alphaF"]]) <-
+          paste0("frailty:", 
+                 names(model_info$frames$mf_Surv)[attr(model_info$terms$terms_Surv, 
+                                                       "specials")$strata], 
+                 levels(model_data$strata)[-1]
+        )
     }
     # drop sigmas that are not needed
     has_sigmas <- initial_values$log_sigmas > -20.0
