@@ -286,7 +286,7 @@ void update_alphaF (const vec &bs_gammas, const vec &gammas, const vec &alphas,
                     const mat &Wlong_H, const mat &Wlong_h, const mat &Wlong_H2,
                     //
                     const bool &recurrent,
-                    const uvec &which_term_H, const uvec &which_term_h,
+                    const field<uvec> &which_term_H, const field<uvec> &which_term_h,
                     const vec &frailty_H, const vec &frailty_h,
                     vec &alphaF, vec &alphaF_H, vec &alphaF_h,
                     vec &scale_alphaF, mat &acceptance_alphaF, mat &res_alphaF,
@@ -295,51 +295,55 @@ void update_alphaF (const vec &bs_gammas, const vec &gammas, const vec &alphas,
                     const bool &shrink_alphaF,
                     const vec &sigmaF,
                     vec &frailtyH_sigmaF_alphaF, vec &frailtyh_sigmaF_alphaF) {
-  vec proposed_alphaF = propose_norm(alphaF, scale_alphaF, 0);
-  vec proposed_alphaF_H(WH_gammas.n_rows, fill::ones);
-  vec proposed_alphaF_h(Wh_gammas.n_rows, fill::ones);
-  proposed_alphaF_H.rows(which_term_H).fill(proposed_alphaF.at(0));
-  proposed_alphaF_h.rows(which_term_h).fill(proposed_alphaF.at(0));
-  vec proposed_frailtyH_sigmaF_alphaF(WH_gammas.n_rows, fill::zeros);
-  vec proposed_frailtyh_sigmaF_alphaF(which_event.n_rows, fill::zeros);
-  proposed_frailtyH_sigmaF_alphaF = frailty_H % proposed_alphaF_H * sigmaF;
-  proposed_frailtyh_sigmaF_alphaF = frailty_h.rows(which_event) % proposed_alphaF_h.rows(which_event) * sigmaF;
-  vec logLik_surv_proposed =
-    log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
-             WH_gammas, Wh_gammas, WH2_gammas,
-             WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
-             log_Pwk, log_Pwk2, indFast_H, indFast_h,
-             which_event, which_right_event, which_left,
-             any_interval, which_interval,
-             recurrent, 
-             proposed_frailtyH_sigmaF_alphaF, proposed_frailtyh_sigmaF_alphaF);
-  double numerator_surv =
-    sum(logLik_surv_proposed) +
-    logPrior_surv(bs_gammas, gammas, alphas, prior_mean_bs_gammas,
-                  prior_Tau_bs_gammas, tau_bs_gammas,
-                  prior_mean_gammas, prior_Tau_gammas, lambda_gammas, tau_gammas, shrink_gammas,
-                  prior_mean_alphas, prior_Tau_alphas, lambda_alphas, tau_alphas, shrink_alphas,
-                  recurrent, proposed_alphaF, prior_mean_alphaF, prior_Tau_alphaF,
-                  lambda_alphaF, tau_alphaF, shrink_alphaF);
-  double log_ratio = numerator_surv - denominator_surv;
-  if (std::isfinite(log_ratio) && exp(log_ratio) > R::runif(0.0, 1.0)) {
-    alphaF = proposed_alphaF;
-    alphaF_H = proposed_alphaF_H;
-    alphaF_h = proposed_alphaF_h;
-    logLik_surv = logLik_surv_proposed;
-    denominator_surv = numerator_surv;
-    acceptance_alphaF.row(it) = 1;
-    frailtyH_sigmaF_alphaF = proposed_frailtyH_sigmaF_alphaF;
-    frailtyh_sigmaF_alphaF = proposed_frailtyh_sigmaF_alphaF;
+  for (uword i = 0; i < alphaF.n_rows; ++i) {
+    vec proposed_alphaF = propose_norm(alphaF, scale_alphaF, i);
+    vec proposed_alphaF_H(WH_gammas.n_rows, fill::ones);
+    vec proposed_alphaF_h(Wh_gammas.n_rows, fill::ones);
+    for (uword j = 0; j < alphaF.n_rows; ++j) {
+      proposed_alphaF_H.rows(which_term_H.at(j)).fill(proposed_alphaF.at(j));
+      proposed_alphaF_h.rows(which_term_h.at(j)).fill(proposed_alphaF.at(j));
+    }
+    vec proposed_frailtyH_sigmaF_alphaF(WH_gammas.n_rows, fill::zeros);
+    vec proposed_frailtyh_sigmaF_alphaF(which_event.n_rows, fill::zeros);
+    proposed_frailtyH_sigmaF_alphaF = frailty_H % proposed_alphaF_H * sigmaF;
+    proposed_frailtyh_sigmaF_alphaF = frailty_h.rows(which_event) % proposed_alphaF_h.rows(which_event) * sigmaF;
+    vec logLik_surv_proposed =
+      log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
+               WH_gammas, Wh_gammas, WH2_gammas,
+               WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
+               log_Pwk, log_Pwk2, indFast_H, indFast_h,
+               which_event, which_right_event, which_left,
+               any_interval, which_interval,
+               recurrent, 
+               proposed_frailtyH_sigmaF_alphaF, proposed_frailtyh_sigmaF_alphaF);
+    double numerator_surv =
+      sum(logLik_surv_proposed) +
+      logPrior_surv(bs_gammas, gammas, alphas, prior_mean_bs_gammas,
+                    prior_Tau_bs_gammas, tau_bs_gammas,
+                    prior_mean_gammas, prior_Tau_gammas, lambda_gammas, tau_gammas, shrink_gammas,
+                    prior_mean_alphas, prior_Tau_alphas, lambda_alphas, tau_alphas, shrink_alphas,
+                    recurrent, proposed_alphaF, prior_mean_alphaF, prior_Tau_alphaF,
+                    lambda_alphaF, tau_alphaF, shrink_alphaF);
+    double log_ratio = numerator_surv - denominator_surv;
+    if (std::isfinite(log_ratio) && exp(log_ratio) > R::runif(0.0, 1.0)) {
+      alphaF = proposed_alphaF;
+      alphaF_H = proposed_alphaF_H;
+      alphaF_h = proposed_alphaF_h;
+      logLik_surv = logLik_surv_proposed;
+      denominator_surv = numerator_surv;
+      acceptance_alphaF.at(it, i) = 1;
+      frailtyH_sigmaF_alphaF = proposed_frailtyH_sigmaF_alphaF;
+      frailtyh_sigmaF_alphaF = proposed_frailtyh_sigmaF_alphaF;
+    }
+    if (it > 19) {
+      scale_alphaF.at(i) =
+        robbins_monro(scale_alphaF.at(i),
+                      acceptance_alphaF.at(it, i), it);
+      
+    }
+    // store results
+    res_alphaF.at(it, i) = alphaF.at(i);
   }
-  if (it > 19) {
-    scale_alphaF.at(0) =
-      robbins_monro(scale_alphaF.at(0),
-                    acceptance_alphaF.at(it, 0), it);
-    
-  }
-  // store results
-  res_alphaF.at(it, 0) = alphaF.at(0);
 }
 
 #endif
