@@ -23,8 +23,8 @@ List mcmc_cpp (List model_data, List model_info, List initial_values,
   uvec which_right_event = join_cols(which_event, which_right);
   uvec which_left = as<uvec>(model_data["which_left"]) - 1;
   uvec which_interval = as<uvec>(model_data["which_interval"]) - 1;
-  uvec which_term_h = as<uvec>(model_data["which_term_h"]) - 1;
-  uvec which_term_H = as<uvec>(model_data["which_term_H"]) - 1;
+  field<uvec> which_term_h = List2Field_uvec(as<List>(model_data["which_term_h"]), true);
+  field<uvec> which_term_H = List2Field_uvec(as<List>(model_data["which_term_H"]), true);
   //
   mat W0_H = as<mat>(model_data["W0_H"]);
   mat W0_h = as<mat>(model_data["W0_h"]);
@@ -198,6 +198,7 @@ List mcmc_cpp (List model_data, List model_info, List initial_values,
   uword n_per_stratum = n_bs_gammas / n_strata;
   uword n_gammas = gammas.n_rows;
   uword n_alphas = alphas.n_rows;
+  uword n_alphaF = alphaF.n_rows;
   uword n_sds = sds.n_rows;
   uword n_L = vec(L(upper_part)).n_rows;
   uword n_sigmas = sigmas.n_rows;
@@ -230,8 +231,8 @@ List mcmc_cpp (List model_data, List model_info, List initial_values,
   mat res_betas(n_iter, n_betas, fill::zeros);
   field<vec> acceptance_betas = create_storage(x_notin_z);
   mat res_logLik(n_iter, n_b, fill::zeros);
-  mat res_alphaF(n_iter, 1, fill::zeros);
-  mat acceptance_alphaF(n_iter, 1, fill::zeros);
+  mat res_alphaF(n_iter, n_alphaF, fill::zeros);
+  mat acceptance_alphaF(n_iter, n_alphaF, fill::zeros);
   mat res_sigmaF(n_iter, 1, fill::zeros);
   mat acceptance_sigmaF(n_iter, 1, fill::zeros);
   mat res_frailty(n_iter, frailty.n_rows, fill::zeros);
@@ -245,7 +246,7 @@ List mcmc_cpp (List model_data, List model_info, List initial_values,
   mat scale_b = mat(n_b,  b_mat.n_cols, fill::ones) * 0.1;
   vec scale_sigmas = create_init_scale(n_sigmas);
   field<vec> scale_betas = create_init_scaleF(x_notin_z);
-  vec scale_alphaF = create_init_scale(1);
+  vec scale_alphaF = create_init_scale(n_alphaF);
   vec scale_sigmaF = create_init_scale(1);
   vec scale_frailty = vec(frailty.n_rows, fill::ones) * 0.5;
   // preliminaries
@@ -282,8 +283,10 @@ List mcmc_cpp (List model_data, List model_info, List initial_values,
   vec alphaF_H(WH_gammas.n_rows, fill::ones);
   vec alphaF_h(Wh_gammas.n_rows, fill::ones);
   if(any_terminal) {
-    alphaF_H.rows(which_term_H).fill(alphaF.at(0));
-    alphaF_h.rows(which_term_h).fill(alphaF.at(0));
+    for (uword j = 0; j < n_strata - 1; ++j) {
+      alphaF_H.rows(which_term_H.at(j)).fill(alphaF.at(j));
+      alphaF_h.rows(which_term_h.at(j)).fill(alphaF.at(j)); 
+    }
   }
   vec frailty_H(WH_gammas.n_rows, fill::zeros);
   vec frailty_h(Wh_gammas.n_rows, fill::zeros);
@@ -681,8 +684,8 @@ arma::vec logLik_jm (List thetas, List model_data, List model_info,
   uvec id_H = as<uvec>(model_data["id_H"]) - 1;
   uvec id_H_fast = create_fast_ind(id_H + 1);
   uvec id_h_fast = create_fast_ind(id_h + 1);
-  uvec which_term_h = as<uvec>(model_data["which_term_h"]) - 1; // recurrence
-  uvec which_term_H = as<uvec>(model_data["which_term_H"]) - 1;
+  field<uvec> which_term_h = List2Field_uvec(as<List>(model_data["which_term_h"]), true);
+  field<uvec> which_term_H = List2Field_uvec(as<List>(model_data["which_term_H"]), true);
   bool any_terminal = !which_term_h.is_empty();
   bool recurrent = as<bool>(model_info["recurrent"]);
   vec alphaF = as<vec>(thetas["alphaF"]);
@@ -786,8 +789,8 @@ arma::mat mlogLik_jm (List res_thetas, arma::mat mean_b_mat, arma::cube post_var
   uvec id_H = as<uvec>(model_data["id_H"]) - 1;
   uvec id_H_fast = create_fast_ind(id_H + 1);
   uvec id_h_fast = create_fast_ind(id_h + 1);
-  uvec which_term_h = as<uvec>(model_data["which_term_h"]) - 1; // recurrence
-  uvec which_term_H = as<uvec>(model_data["which_term_H"]) - 1;
+  field<uvec> which_term_h = List2Field_uvec(as<List>(model_data["which_term_h"]), true);
+  field<uvec> which_term_H = List2Field_uvec(as<List>(model_data["which_term_H"]), true);
   bool any_terminal = !which_term_h.is_empty();
   bool recurrent = as<bool>(model_info["recurrent"]);
   mat alphaF = trans(as<mat>(res_thetas["alphaF"]));
